@@ -25,10 +25,11 @@
  */
 
 #include <config.h>
-#include <glib/gprintf.h>
 #include <math.h>
 #include <string.h>
+#undef GTK_DISABLE_DEPRECATED
 #include "gtkprogress.h" 
+#define GTK_DISABLE_DEPRECATED
 #include "gtkprivate.h" 
 #include "gtkintl.h"
 #include "gtkalias.h"
