@@ -27,7 +27,6 @@
 #undef GTK_DISABLE_DEPRECATED
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtksignal.h"
@@ -36,6 +35,8 @@
 #define GTK_ENABLE_BROKEN
 #include "gtktree.h"
 #include "gtktreeitem.h"
+#include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   SELECTION_CHANGED,
@@ -98,6 +99,7 @@ gtk_tree_get_type (void)
         (GtkClassInitFunc) NULL,
       };
       
+      I_("GtkTree");
       tree_type = gtk_type_unique (gtk_container_get_type (), &tree_info);
     }
   
@@ -141,14 +143,14 @@ gtk_tree_class_init (GtkTreeClass *class)
   class->unselect_child = gtk_real_tree_unselect_child;
 
   tree_signals[SELECTION_CHANGED] =
-    gtk_signal_new ("selection_changed",
+    gtk_signal_new (I_("selection_changed"),
 		    GTK_RUN_FIRST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTreeClass, selection_changed),
 		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   tree_signals[SELECT_CHILD] =
-    gtk_signal_new ("select_child",
+    gtk_signal_new (I_("select_child"),
 		    GTK_RUN_FIRST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTreeClass, select_child),
@@ -156,7 +158,7 @@ gtk_tree_class_init (GtkTreeClass *class)
 		    GTK_TYPE_NONE, 1,
 		    GTK_TYPE_WIDGET);
   tree_signals[UNSELECT_CHILD] =
-    gtk_signal_new ("unselect_child",
+    gtk_signal_new (I_("unselect_child"),
 		    GTK_RUN_FIRST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTreeClass, unselect_child),
@@ -1129,3 +1131,6 @@ gtk_tree_set_view_lines (GtkTree       *tree,
   
   tree->view_line = flag;
 }
+
+#define __GTK_TREE_C__
+#include "gtkaliasdef.c"
